#include <Arduino.h> //needed for Serial.println
#include <string.h> //needed for memcpy
#include <stdlib.h>
#include <SoftwareSerial.h>

//                        Rx  Tx
SoftwareSerial debugSerial(8, 9);
enum retType {
  ERROR = 0,
  OK = 1,
  NORESP = 2    };
  
class Esp8266{
public:

  Esp8266(int);
  int isOk();
  int setModeWifiClient();
  int connectToWifi(String ssid, String pwd);
  int httpGet(String ip, String path, int port);
  int httpClose();
  void On();
  void Off();
private:
  int wifiPwrCtrlPin; // Control power to the wifi module
  String buf;
  int debug;
  void readToBuf(int timeOutMsec);
  void serialFlush();
  int isRespOk(int timeOutMsec);
  void serialPrintln(String str);

};
Esp8266::Esp8266(int powerCtrlPin)
{
  wifiPwrCtrlPin = powerCtrlPin;
  debugSerial.begin(9600);
  debug = 1;
  //Set the power control pin of wifi module to output
  pinMode(wifiPwrCtrlPin, OUTPUT); 
}
void Esp8266::On()
{
  digitalWrite(wifiPwrCtrlPin, HIGH);
  delay(1000);
  serialFlush();
}
void Esp8266::Off()
{
  digitalWrite(wifiPwrCtrlPin, LOW);
}
// Flush all data in serial in buffer
void Esp8266::serialFlush()
{
  while(Serial.available() > 0) {
    char t = Serial.read();
  }
}   
void Esp8266::serialPrintln(String str)
{

  if (debug)
  {
    //Mega
    //Serial.print("Arduino>");
    //Serial.println(str);
    //Soft Serial
    debugSerial.print("Arduino>");
    debugSerial.println(str);
  }
  serialFlush();
  Serial.println(str);
}
int Esp8266::httpClose()
{
  int resp;
  //AT+CIPCLOSE
  String atTcpPortCloseConnCmd = "AT+CIPCLOSE";
  serialPrintln(atTcpPortCloseConnCmd);
  resp = isRespOk(1500);
  if (resp != OK )
    return resp;  
}
int Esp8266::httpGet(String ip, String path, int port=80)
{
  int resp;
  String atHttpGetCmd = "GET /"+path+" HTTP/1.0\r\n\r\n";
  //AT+CIPSTART="TCP","192.168.20.200",80
  String atTcpPortConnectCmd = "AT+CIPSTART=\"TCP\",\""+ip+"\","+port+"";
  serialPrintln(atTcpPortConnectCmd);


  resp = isRespOk(1500);
  if (resp != OK )
    return resp;

  
  int len = atHttpGetCmd.length();
  //AT+CIPSEND=40
  String atSendCmd = "AT+CIPSEND=";
  atSendCmd += len;
  serialPrintln(atSendCmd);
  
  resp = isRespOk(2500);
  if (resp != OK )
  {
    return resp;
  }

  //GET /invoice/ HTTP/1.0\r\n\r\n
  serialPrintln(atHttpGetCmd);
  resp = isRespOk(2500);
  return resp;  
}

int Esp8266::connectToWifi(String ssid, String pwd)
{
  String atConnectCmd = "AT+CWJAP=\""+ssid+"\",\""+pwd+"\"";
  serialPrintln(atConnectCmd);
  return isRespOk(5500);
}

int Esp8266::setModeWifiClient()
{
  serialPrintln("AT+CWMODE=1");
  return isRespOk(1500);
}

int Esp8266::isRespOk(int timeOutMsec=1500)
{
  //Memory optimization no locals, buffer should have the data from serial
  //Was seeing null strings when buf (~600bytes)was returned, likely due to low memory
  readToBuf(timeOutMsec);
  
  if (buf.indexOf("OK") != -1 )
  {
    return OK;
  }
  else if (buf.indexOf("ERROR") != -1 && buf.indexOf("Error") != -1)
  {
    return ERROR;
  }
  else if (buf.indexOf(">") != -1)
  {
    return OK;
  }
  else if (buf.indexOf("no change") != -1)
  {
    return OK;
  }
  else
  {
    return NORESP;
  }
}

int Esp8266::isOk(){
  serialPrintln("AT");
  return isRespOk();
}

/* Read Serial data to buf until timeOutMsecs expire
 */
void Esp8266::readToBuf(int timeOutMsec)
{ 
  buf = String(""); //clear buffer
  unsigned long curr_time = millis();
  while(1)
  {
    if (Serial.available()) 
    {
      char c = Serial.read();
      buf += c;    
    }
    //If more than 1 secs passed return
    if (millis() > curr_time + timeOutMsec)
    {
      //For debugging
      if (debug)
      {
        debugSerial.print("ESP8266>");
        debugSerial.println(buf);

        //For Mega
        //Serial.print("ESP8266>");
        //Serial.println(buf);
      }
      //Buf has all it needs
      return;

    }
  }
}



